[PATCH] [MTD] OneNAND: Select correct chip's bufferRAM for DDP
Adrian Hunter
ext-adrian.hunter at nokia.com
Thu Feb 8 05:04:54 EST 2007
>From 9cb655eddb756a18a7deadc08fed9abbf934c13c Mon Sep 17 00:00:00 2001
From: Adrian Hunter <ext-adrian.hunter at nokia.com>
Date: Thu, 8 Feb 2007 10:28:08 +0200
Subject: [MTD] OneNAND: Select correct chip's bufferRAM for DDP
OneNAND double-density package (DDP) has two chips, each with
their own bufferRAM. The driver will skip loading data from
the NAND core if the data can be found in a bufferRAM, however
in that case, the correct chip's bufferRAM must be selected
before reading from bufferRAM.
Signed-off-by: Adrian Hunter <ext-adrian.hunter at nokia.com>
---
drivers/mtd/onenand/onenand_base.c | 24 ++++++++++++++++--------
1 files changed, 16 insertions(+), 8 deletions(-)
diff --git a/drivers/mtd/onenand/onenand_base.c b/drivers/mtd/onenand/onenand_base.c
index cf66c93..162a373 100644
--- a/drivers/mtd/onenand/onenand_base.c
+++ b/drivers/mtd/onenand/onenand_base.c
@@ -577,7 +577,7 @@ static int onenand_write_bufferram(struc
static int onenand_check_bufferram(struct mtd_info *mtd, loff_t addr)
{
struct onenand_chip *this = mtd->priv;
- int blockpage;
+ int blockpage, found = 0;
unsigned int i;
blockpage = (int) (addr >> this->page_shift);
@@ -585,16 +585,24 @@ static int onenand_check_bufferram(struc
/* Is there valid data? */
i = ONENAND_CURRENT_BUFFERRAM(this);
if (this->bufferram[i].blockpage == blockpage)
- return 1;
+ found = 1;
+ else {
+ /* Check another BufferRAM */
+ i = ONENAND_NEXT_BUFFERRAM(this);
+ if (this->bufferram[i].blockpage == blockpage) {
+ ONENAND_SET_NEXT_BUFFERRAM(this);
+ found = 1;
+ }
+ }
- /* Check another BufferRAM */
- i = ONENAND_NEXT_BUFFERRAM(this);
- if (this->bufferram[i].blockpage == blockpage) {
- ONENAND_SET_NEXT_BUFFERRAM(this);
- return 1;
+ if (found && ONENAND_IS_DDP(this)) {
+ /* Select DataRAM for DDP */
+ int block = (int) (addr >> this->erase_shift);
+ int value = onenand_bufferram_address(this, block);
+ this->write_word(value, this->base + ONENAND_REG_START_ADDRESS2);
}
- return 0;
+ return found;
}
/**
--
1.4.3
More information about the linux-mtd
mailing list